/*
 *  /MathJax-v2/localization/cs/FontWarnings.js
 *
 *  Copyright (c) 2009-2018 The MathJax Consortium
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

MathJax.Localization.addTranslation("cs","FontWarnings",{version:"2.7.9",isLoaded:true,strings:{webFont:"MathJax pou\u017E\u00EDv\u00E1 k zobrazen\u00ED matematiky na t\u00E9to str\u00E1nce webov\u00E1 p\u00EDsma. T\u011Bm trv\u00E1 sta\u017Een\u00ED n\u011Bjakou dobu, tak\u017Ee str\u00E1nka by se vykreslila rychleji, pokud byste si tato matematick\u00E1 p\u00EDsma nainstalovali p\u0159\u00EDmo do syst\u00E9mov\u00E9 slo\u017Eky p\u00EDsem.",imageFonts:"MathJax pou\u017E\u00EDv\u00E1 m\u00EDsto m\u00EDstn\u00EDch nebo webov\u00FDch p\u00EDsem obr\u00E1zkov\u00E9 p\u00EDsmo. Proto bude vykreslov\u00E1n\u00ED pomalej\u0161\u00ED, ne\u017E je obvykl\u00E9, a p\u0159i tisku na Va\u0161\u00ED tisk\u00E1rn\u011B mo\u017En\u00E1 nebude matematika vykreslena na nejlep\u0161\u00ED rozli\u0161en\u00ED.",noFonts:"MathJax nedok\u00E1\u017Ee naj\u00EDt p\u00EDsmo, kter\u00FDm by vykreslil matematiku, a obr\u00E1zkov\u00E1 p\u00EDsma nejsou k dispozici, tak\u017Ee se uch\u00FDl\u00ED k b\u011B\u017En\u00FDm znak\u016Fm Unicode v nad\u011Bji, \u017Ee je v\u00E1\u0161 prohl\u00ED\u017Ee\u010D dok\u00E1\u017Ee zobrazit. N\u011Bkter\u00E9 znaky se mo\u017En\u00E1 nezobraz\u00ED spr\u00E1vn\u011B, nebo dokonce v\u016Fbec.",webFonts:"V\u011Bt\u0161ina modern\u00EDch prohl\u00ED\u017Ee\u010D\u016F umo\u017E\u0148uje sta\u017Een\u00ED p\u00EDsem p\u0159es web. Aktualizace na nov\u011Bj\u0161\u00ED verzi prohl\u00ED\u017Ee\u010De (nebo zm\u011Bna prohl\u00ED\u017Ee\u010De) by mohla zlep\u0161it zobrazen\u00ED matematiky na t\u00E9to str\u00E1nce.",fonts:"MathJax m\u016F\u017Ee pou\u017E\u00EDvat bu\u010F [p\u00EDsma STIX](%1) nebo [MathJaxov\u00E1 TeXov\u00E1 p\u00EDsma](%2). St\u00E1hn\u011Bte a nainstalujte si jedny z nich, abyste si MathJax u\u017Eili l\u00E9pe.",STIXPage:"Tato str\u00E1nka je vytvo\u0159ena pro pou\u017Eit\u00ED s [p\u00EDsmy STIX](%1). St\u00E1hn\u011Bte a nainstalujte si tato p\u00EDsma, abyste si l\u00E9pe u\u017Eili MathJax.",TeXPage:"Tato str\u00E1nka je vytvo\u0159ena pro pou\u017Eit\u00ED s [MathJaxov\u00FDmi TeXov\u00FDmi p\u00EDsmy](%1). Pro lep\u0161\u00ED vyu\u017E\u00EDv\u00E1n\u00ED MathJaxu si st\u00E1hn\u011Bte a nainstalujte tato p\u00EDsma."}});MathJax.Ajax.loadComplete("[MathJax]/localization/cs/FontWarnings.js");
